<?php
/**
 * Plugin Name: Members Only Menu Plugin
 * Plugin URI: http://brandonwamboldt.ca/2010/08/members-only-menu-plugin/
 * Author: Brandon Wamboldt
 * Author URI: http://brandonwamboldt.ca/
 * Version: 1.1
 * Description: A plugin that allows you to create a page, restrict that page to authenticated users, and still have the ability to add that page to the WordPress menu. Note: The items will only show up for authenticated users
 */

add_action( 'wp'             , array( 'MembersOnlyMenuPlugin' , 'check_for_members_only' ) );
add_action( 'add_meta_boxes' , array( 'MembersOnlyMenuPlugin' , 'add_members_only_meta_box' ) );
add_action( 'save_post'      , array( 'MembersOnlyMenuPlugin' , 'members_only_save_postdata' ) );

add_filter( 'wp_nav_menu_args' , array( 'MembersOnlyMenuPlugin' , 'wp_nav_menu_args' ) );

/**
 * A custom walker that checks to see if the page is members only then
 * checks for authentication
 * 
 * @author brandon
 * @package WordPress
 * @subpackage MembersOnlyMenuPlugin
 */
class Members_Only_Nav_Menu_Walker extends Walker_Nav_Menu {
    function start_el(&$output, $item, $depth, $args) {
        global $wp_query;

        $indent = ( $depth ) ? str_repeat( "\t", $depth ) : '';

        $class_names = $value = '';

        $classes = empty( $item->classes ) ? array() : (array) $item->classes;

        $class_names = join( ' ', apply_filters( 'nav_menu_css_class', array_filter( $classes ), $item ) );
        $class_names = ' class="' . esc_attr( $class_names ) . '"';

        // Check to see if the user is logged in
        if ( !get_post_meta( $item->object_id , '_members_only_menu' , true ) || is_user_logged_in() ) {
            $output .= $indent . '<li id="menu-item-'. $item->ID . '"' . $value . $class_names .'>';
        }

        $attributes  = ! empty( $item->attr_title ) ? ' title="'  . esc_attr( $item->attr_title ) .'"' : '';
        $attributes .= ! empty( $item->target )     ? ' target="' . esc_attr( $item->target     ) .'"' : '';
        $attributes .= ! empty( $item->xfn )        ? ' rel="'    . esc_attr( $item->xfn        ) .'"' : '';
        $attributes .= ! empty( $item->url )        ? ' href="'   . esc_attr( $item->url        ) .'"' : '';

        $item_output = $args->before;
        $item_output .= '<a'. $attributes .'>';
        $item_output .= $args->link_before . apply_filters( 'the_title', $item->title, $item->ID ) . $args->link_after;
        $item_output .= '</a>';
        $item_output .= $args->after;

        // Check to see if the user is logged in
        if ( !get_post_meta( $item->object_id, '_members_only_menu' , true ) || is_user_logged_in() ) {
            $output .= apply_filters( 'walker_nav_menu_start_el', $item_output, $item, $depth, $args );
        }
    }

    function end_el(&$output, $item, $depth) {
        
        // Check to see if the user is logged in
        if ( !get_post_meta( $item->object_id, '_members_only_menu' , true ) || is_user_logged_in() ) {
            $output .= "</li>\n";
        }
    }
}

/**
 * The main plugin
 * 
 * @author brandon
 * @package WordPress
 * @subpackage MembersOnlyMenuPlugin
 */
class MembersOnlyMenuPlugin {

    /**
     * This executes right after the post data is set, and 
     * checks if some post meta is set. If it is, we redirect 
     * to the login page (Only if the user isn't logged in).
     */
    public static function check_for_members_only() {
        global $post;
    
        if ( get_post_meta( $post->ID , '_members_only_menu' , true ) && !is_user_logged_in() ) {
            header( 'Location: ' . get_bloginfo( 'wpurl' ) . '/wp-login.php' );
            exit();
        }
    }
    
    /**
     * Add a meta box to the editor for pages
     */
    public static function add_members_only_meta_box() {
        add_meta_box( 'add_members_only_meta' , 'Members Only' , array( 'MembersOnlyMenuPlugin' , 'add_members_only_meta' ) , 'page' , 'side' , 'high' );
    }

    /**
     * This generates the HTML for the meta box (A nonce and a checkbox)
     */
    public static function add_members_only_meta() {
        global $post;
    
        $meta_box_value = get_post_meta( $post->ID , '_members_only_menu' , true ); 
    
        if ( $meta_box_value == 'true' ) {
            $meta_box_value = 'checked="checked"';
        }
    
        echo '<input type="hidden" name="members_only_nonce" id="members_only_nonce" value="' . wp_create_nonce( 'members_only' ) . '" />';
        echo '<input type="checkbox" name="members_only" id="members_only" value="true" '.$meta_box_value .' /> ' . __('Restrict To Members?');
    }
    
    /**
     * Saves meta data to save our members only value
     * 
     * @param int $post_id The ID of the page we are saving
     */
    public static function members_only_save_postdata( $post_id )
    {
        /*
         * Verify this came from the our screen and with proper authorization,
         * because save_post can be triggered at other times
         */
        if ( !isset( $_POST['members_only_nonce'] ) ) return $post_id;
        
        if ( !wp_verify_nonce( $_POST['members_only_nonce'], 'members_only' ) ) {
            return $post_id;
        }
    
        /*
         * verify if this is an auto save routine. If it is our form has not been submitted, 
         * so we dont want to do anything
         */
        if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE )
            return $post_id;

        // Save
        if ( isset( $_POST['members_only'] ) && $_POST['members_only'] == 'true' ) {
            update_post_meta( $post_id , '_members_only_menu' , 'true' );
        } else {
            delete_post_meta( $post_id , '_members_only_menu' );
        }
    
        return $post_id;
    }
    
    public static function wp_nav_menu_args( $args ) {
        $args['walker'] = new Members_Only_Nav_Menu_Walker();
        return $args;
    }
}
